@@ -12,7 +12,6 @@ class ToolTemplate():
 
 	def __init__(self, tool_id, label, icon_name, window, **kwargs):
 		self.id = tool_id
-		self.implements_panel = False
 		self.accept_selection = False
 		self.use_color = True
 		self.menu_id = 0
@@ -24,6 +23,7 @@ class ToolTemplate():
 		self.has_ongoing_operation = False # TODO
 		self.window = window
 		self.build_row()
+		self.try_build_panel()
 
 	############################################################################
 	# Actions ##################################################################
@@ -55,12 +55,11 @@ class ToolTemplate():
 	def show_error(self, error_text):
 		self.window.prompt_message(True, error_text)
 
-	def show_panel(self, visibility):
-		if self.implements_panel:
-			self.window.bottom_panel.set_visible(not visibility)
-			self.bottom_panel.set_visible(visibility)
-		else:
-			self.window.bottom_panel.set_visible(True)
+	def try_build_panel(self):
+		pass
+
+	def build_bottom_panel(self):
+		return None
 
 	def add_item_to_menu(self, tools_menu):
 		tools_menu.append(self.label, 'win.active_tool::' + self.id)
@@ -82,8 +81,9 @@ class ToolTemplate():
 	def build_row(self):
 		"""Build the GtkRadioButton for the sidebar. This method stores it as
 		'self.row', but does not pack it in the bar, and does not return it."""
-		self.row = Gtk.RadioButton(draw_indicator=False, \
-		                   tooltip_text=self.label, relief=Gtk.ReliefStyle.NONE)
+		self.row = Gtk.RadioButton(relief=Gtk.ReliefStyle.NONE, \
+		                        draw_indicator=False, valign=Gtk.Align.CENTER, \
+		                                                tooltip_text=self.label)
 		self.row.set_detailed_action_name('win.active_tool::' + self.id)
 		self.label_widget = Gtk.Label(label=self.label)
 		# if self.window.decorations == 'csd-eos': # XXX faisable mais bug laid
@@ -133,10 +133,16 @@ class ToolTemplate():
 		pass
 
 	def apply_operation(self, operation):
+		self.get_image().add_pixbuf_to_history()
+		self.simple_apply_operation(operation)
+		self.get_image().update_actions_state()
+		self.get_image().update_history_sensitivity()
+
+	def simple_apply_operation(self, operation):
+		"""Simpler apply_operation, for the "rebuild from history" method."""
 		self.do_tool_operation(operation)
+		self.get_image().add_to_history(operation)
 		self.non_destructive_show_modif()
-		self.apply_to_pixbuf()
-		self.get_image().add_operation_to_history(operation)
 
 	############################################################################
 	# Selection ################################################################
@@ -174,32 +180,30 @@ class ToolTemplate():
 	def restore_pixbuf(self):
 		self.get_image().use_stable_pixbuf()
 
-	def apply_to_pixbuf(self):
-		self.get_image().on_tool_finished()
-
 	############################################################################
 	# Signals handling #########################################################
 
-	def on_press_on_area(self, area, event, surface, tool_width, lc, rc, e_x, e_y):
+	def on_press_on_area(self, event, surface, event_x, event_y):
 		pass
 
-	def on_motion_on_area(self, area, event, surface, event_x, event_y):
+	def on_motion_on_area(self, event, surface, event_x, event_y):
 		pass
 
 	def on_unclicked_motion_on_area(self, event, surface):
 		pass
 
-	def on_release_on_area(self, area, event, surface, event_x, event_y):
+	def on_release_on_area(self, event, surface, event_x, event_y):
 		pass
 
 	def on_draw(self, area, cairo_context):
 		if not self.accept_selection:
 			return
-		# Very basic implementation
+		# Basic implementation, tools should do it better to fit their needs
 		if self.selection_is_active():
 			self.get_selection().show_selection_on_surface(cairo_context, True)
-			utilities_show_overlay_on_context(cairo_context, \
-			                self.get_image().get_dragged_selection_path(), True)
+			dragged_path = self.get_selection().get_path_with_scroll()
+			utilities_show_overlay_on_context(cairo_context, dragged_path, True)
 
 	############################################################################
 ################################################################################
+
