@@ -82,7 +82,7 @@ class DrawingAdaptativeHeaderBar():
 		if incoherent:
 			self.set_compact(not self.is_narrow)
 
-	def set_compact(self, state):
+	def set_compact(self, state): # TODO state as an int
 		if state:
 			self.main_menu_btn.set_menu_model(self.long_main_menu)
 		else:
