@@ -16,7 +16,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, GLib, Pango
-from .gi_composites import GtkTemplate
 import cairo
 
 from .utilities import utilities_save_pixbuf_at
@@ -29,19 +28,18 @@ class DrawingMotionBehavior():
 
 ################################################################################
 
-@GtkTemplate(ui='/com/github/maoschanz/drawing/ui/image.ui')
+@Gtk.Template(resource_path='/com/github/maoschanz/drawing/ui/image.ui')
 class DrawingImage(Gtk.Box):
 	__gtype_name__ = 'DrawingImage'
 
-	drawing_area = GtkTemplate.Child()
-	h_scrollbar = GtkTemplate.Child()
-	v_scrollbar = GtkTemplate.Child()
+	drawing_area = Gtk.Template.Child()
+	h_scrollbar = Gtk.Template.Child()
+	v_scrollbar = Gtk.Template.Child()
 
 	CLOSING_PRECISION = 10
 
 	def __init__(self, window, **kwargs):
 		super().__init__(**kwargs)
-		self.init_template()
 		self.window = window
 
 		self.gfile = None
@@ -70,6 +68,10 @@ class DrawingImage(Gtk.Box):
 		# For the cursor
 		self.drawing_area.connect('enter-notify-event', self.on_enter_image)
 		self.drawing_area.connect('leave-notify-event', self.on_leave_image)
+		# [Option] automatic zoom level
+		self.drawing_area.connect('size-allocate', self.set_opti_zoom_level)
+
+		self.ctrl_to_zoom = self.window._settings.get_string('zoom-behavior') == 'ctrl'
 
 	############################################################################
 	# Image initialization #####################################################
@@ -77,6 +79,7 @@ class DrawingImage(Gtk.Box):
 	def init_image(self):
 		"""Part of the initialization common to both a new blank image and an
 		opened image."""
+		self._is_pressed = False
 
 		# Zoom and scroll initialization
 		self.zoom_level = 1.0
@@ -87,11 +90,14 @@ class DrawingImage(Gtk.Box):
 		self.press2_y = 0.0
 		self.drag_scroll_x = 0.0
 		self.drag_scroll_y = 0.0
+		self.zoom_is_auto = self.window._settings.get_boolean('auto-zoom')
+		# TODO peut être updaté quand la valeur de la clef change
 
 		# Selection initialization
 		self.selection = DrawingSelectionManager(self)
 		
 		# History initialization
+		self.previous_pixbuf = None
 		self.undo_history = []
 		self.redo_history = []
 		self._is_saved = True
@@ -120,17 +126,13 @@ class DrawingImage(Gtk.Box):
 		self.restore_first_pixbuf()
 
 	def try_load_pixbuf(self, pixbuf):
-		if not pixbuf.get_has_alpha() and self.window._settings.get_boolean('add-alpha'):
+		if not pixbuf.get_has_alpha():
 			pixbuf = pixbuf.add_alpha(False, 0, 0, 0)
 		self.initial_operation = {
 			'tool_id': None,
 			'pixbuf': pixbuf,
-			'red': 0.0,
-			'green': 0.0,
-			'blue': 0.0,
-			'alpha': 0.0,
-			'width': pixbuf.get_width(),
-			'height': pixbuf.get_height()
+			'red': 0.0, 'green': 0.0, 'blue': 0.0, 'alpha': 0.0,
+			'width': pixbuf.get_width(), 'height': pixbuf.get_height()
 		}
 		self.main_pixbuf = pixbuf
 		self.init_image()
@@ -164,23 +166,8 @@ class DrawingImage(Gtk.Box):
 
 	def try_load_file(self, gfile):
 		self.gfile = gfile
-		self.main_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
 		pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.get_file_path())
-		if not pixbuf.get_has_alpha() and self.window._settings.get_boolean('add-alpha'):
-			pixbuf = pixbuf.add_alpha(False, 0, 0, 0)
-		self.initial_operation = {
-			'tool_id': None,
-			'pixbuf': pixbuf,
-			'red': 0.0,
-			'green': 0.0,
-			'blue': 0.0,
-			'alpha': 0.0,
-			'width': pixbuf.get_width(),
-			'height': pixbuf.get_height()
-		}
-		self.init_image()
-		self.restore_first_pixbuf()
-		self.update_title()
+		self.try_load_pixbuf(pixbuf)
 
 	############################################################################
 	# Image title and tab management ###########################################
@@ -241,33 +228,40 @@ class DrawingImage(Gtk.Box):
 		else:
 			return self.gfile.get_path()
 
-	############################################################################
-	# Wrappers for window methods related to tools #############################
-
-	def active_tool(self):
-		return self.window.active_tool()
-
-	def get_right_rgba(self):
-		return self.window.color_popover_r.color_widget.get_rgba()
-
-	def get_left_rgba(self):
-		return self.window.color_popover_l.color_widget.get_rgba()
-
 	############################################################################
 	# History management #######################################################
 
 	def try_undo(self, *args):
-		self.active_tool().cancel_ongoing_operation()
-		if len(self.undo_history) != 0:
-			self.redo_history.append(self.undo_history.pop())
 		# TODO implement operation_is_ongoing
 		# if self.window.operation_is_ongoing():
 		# 	self.active_tool().cancel_ongoing_operation()
 		# elif len(self.undo_history) != 0:
 		# 	self.redo_history.append(self.undo_history.pop())
+		self.active_tool().cancel_ongoing_operation()
+		if len(self.undo_history) != 0:
+			self.redo_history.append(self.undo_history.pop())
+		if self.previous_pixbuf is not None:
+			self.main_pixbuf = self.previous_pixbuf
+			self.previous_pixbuf = None
+			self.use_stable_pixbuf()
+			self.update()
+			self.update_history_sensitivity()
+		else:
+			self.rebuild_from_history()
 
 	def try_redo(self, *args):
-		self.undo_history.append(self.redo_history.pop())
+		operation = self.redo_history.pop()
+		self.window.tools[operation['tool_id']].apply_operation(operation)
+
+	def rebuild_from_history(self):
+		self.restore_first_pixbuf()
+		history = self.undo_history.copy()
+		self.undo_history = []
+		for op in history:
+			# print(op)
+			self.window.tools[op['tool_id']].simple_apply_operation(op)
+		self.update()
+		self.update_history_sensitivity()
 
 	def update_history_sensitivity(self):
 		can_undo = ( len(self.undo_history) != 0 ) or self.window.operation_is_ongoing()
@@ -288,18 +282,22 @@ class DrawingImage(Gtk.Box):
 			redo_label = None
 		self.window.update_history_actions_labels(undo_label, redo_label)
 
-	def add_operation_to_history(self, operation):
+	def add_pixbuf_to_history(self):
+		# self.previous_pixbuf = self.main_pixbuf.copy()
+		pass # TODO
+
+	def add_to_history(self, operation):
+		self.set_surface_as_stable_pixbuf()
+		# print('add_operation_to_history')
+		# print(operation['tool_id'])
+		# if operation['tool_id'] == 'select':
+		#	print(operation['operation_type'])
+		# print('-----------------------------------')
 		self._is_saved = False
 		self.undo_history.append(operation)
-		self.update_history_sensitivity()
 
-	def on_tool_finished(self): # XXX malgré son nom, cette méthode est appelée
-		                        # (uniquement) par [Tool].apply_to_pixbuf
-		#self.redo_history = []
-		self.update_history_sensitivity()
-		self.update()
-		self.set_surface_as_stable_pixbuf() # XXX ici ou dans [Tool].apply_operation ?
-		self.update_actions_state()
+	############################################################################
+	# Misc ? ###################################################################
 
 	def set_action_sensitivity(self, action_name, state):
 		self.window.lookup_action(action_name).set_enabled(state)
@@ -314,6 +312,9 @@ class DrawingImage(Gtk.Box):
 		self.set_action_sensitivity('new_tab_selection', state)
 		self.active_tool().update_actions_state()
 
+	def active_tool(self):
+		return self.window.active_tool()
+
 	############################################################################
 	# Drawing area, main pixbuf, and surface management ########################
 
@@ -330,6 +331,8 @@ class DrawingImage(Gtk.Box):
 		"""Signal callback. Executed when a mouse button is pressed on
 		self.drawing_area, if the button is the mouse wheel the colors are
 		exchanged, otherwise the signal is transmitted to the selected tool."""
+		if self._is_pressed:
+			return
 		event_x, event_y = self.get_event_coords(event)
 		if event.button == 2:
 			self.motion_behavior = DrawingMotionBehavior.SLIP
@@ -339,9 +342,8 @@ class DrawingImage(Gtk.Box):
 			self.drag_scroll_y = event_y
 			return
 		self.motion_behavior = DrawingMotionBehavior.DRAW
-		self.active_tool().on_press_on_area(area, event, self.surface, \
-		                            self.window.thickness_spinbtn.get_value(), \
-		          self.get_left_rgba(), self.get_right_rgba(), event_x, event_y)
+		self.active_tool().on_press_on_area(event, self.surface, event_x, event_y)
+		self._is_pressed = True
 
 	def on_motion_on_area(self, area, event):
 		"""Signal callback. Executed when the mouse pointer moves upon
@@ -354,7 +356,8 @@ class DrawingImage(Gtk.Box):
 			# XXX ça apprécierait sans doute d'avoir direct les bonnes coordonnées ?
 			self.active_tool().on_unclicked_motion_on_area(event, self.surface)
 		elif self.motion_behavior == DrawingMotionBehavior.DRAW:
-			self.active_tool().on_motion_on_area(area, event, self.surface, event_x, event_y)
+			# implicitely impossible if not self._is_pressed
+			self.active_tool().on_motion_on_area(event, self.surface, event_x, event_y)
 			self.update()
 		else: # self.motion_behavior == DrawingMotionBehavior.SLIP:
 			delta_x = int(self.drag_scroll_x - event_x)
@@ -370,13 +373,14 @@ class DrawingImage(Gtk.Box):
 		if self.motion_behavior == DrawingMotionBehavior.SLIP:
 			if abs(self.press2_x - self.drag_scroll_x) < self.CLOSING_PRECISION \
 			and abs(self.press2_y - self.drag_scroll_y) < self.CLOSING_PRECISION:
-				self.window.exchange_colors()
+				self.window.on_middle_click(event)
 			self.motion_behavior = DrawingMotionBehavior.HOVER
 			return
 		self.motion_behavior = DrawingMotionBehavior.HOVER
 		event_x, event_y = self.get_event_coords(event)
-		self.active_tool().on_release_on_area(area, event, self.surface, event_x, event_y)
+		self.active_tool().on_release_on_area(event, self.surface, event_x, event_y)
 		self.window.set_picture_title()
+		self._is_pressed = False
 
 	def update(self):
 		self.drawing_area.queue_draw()
@@ -415,17 +419,12 @@ class DrawingImage(Gtk.Box):
 
 	def set_main_pixbuf(self, new_pixbuf):
 		if new_pixbuf is None:
+			# FIXME wtf, throw something maybe???
 			return False
 		else:
 			self.main_pixbuf = new_pixbuf
 			return True
 
-	def get_widget_width(self):
-		return self.drawing_area.get_allocated_width()
-
-	def get_widget_height(self):
-		return self.drawing_area.get_allocated_height()
-
 	############################################################################
 	# Temporary pixbuf management ##############################################
 
@@ -444,6 +443,42 @@ class DrawingImage(Gtk.Box):
 		self.use_stable_pixbuf()
 		self.update()
 
+	############################################################################
+	# Interaction with the minimap #############################################
+
+	def get_widget_width(self):
+		return self.drawing_area.get_allocated_width()
+
+	def get_widget_height(self):
+		return self.drawing_area.get_allocated_height()
+
+	def get_mini_pixbuf(self, preview_size):
+		mpb_width = self.get_pixbuf_width()
+		mpb_height = self.get_pixbuf_height()
+		if mpb_height > mpb_width:
+			mw = preview_size * (mpb_width/mpb_height)
+			mh = preview_size
+		else:
+			mw = preview_size
+			mh = preview_size * (mpb_height/mpb_width)
+		return self.main_pixbuf.scale_simple(mw, mh, GdkPixbuf.InterpType.TILES)
+
+	def get_show_overlay(self):
+		mpb_width = self.get_pixbuf_width()
+		mpb_height = self.get_pixbuf_height()
+		show_x = self.get_widget_width() < mpb_width * self.zoom_level
+		show_y = self.get_widget_height() < mpb_height * self.zoom_level
+		show_overlay = show_x or show_y
+		return show_overlay
+
+	def get_minimap_ratio(self, mini_width):
+		return mini_width/self.get_pixbuf_width()
+
+	def get_visible_size(self):
+		visible_width = int(self.get_widget_width() / self.zoom_level)
+		visible_height = int(self.get_widget_height() / self.zoom_level)
+		return visible_width, visible_height
+
 	############################################################################
 	# Scroll and zoom levels ###################################################
 
@@ -456,10 +491,13 @@ class DrawingImage(Gtk.Box):
 		return event_x, event_y
 
 	def on_scroll_on_area(self, area, event):
-		self.add_deltas(event.delta_x, event.delta_y, 10)
-
-	def get_dragged_selection_path(self):
-		return self.selection.get_path_with_scroll(self.scroll_x, self.scroll_y)
+		# TODO https://lazka.github.io/pgi-docs/index.html#Gdk-3.0/classes/EventScroll.html#Gdk.EventScroll
+		ctrl_is_used = (event.state & Gdk.ModifierType.CONTROL_MASK) == Gdk.ModifierType.CONTROL_MASK
+		if ctrl_is_used == self.ctrl_to_zoom:
+			event_x, event_y = self.get_event_coords(event)
+			self.zoom_to_point(event.delta_x, event.delta_y, event.x, event.y)
+		else:
+			self.add_deltas(event.delta_x, event.delta_y, 10)
 
 	def on_scrollbar_value_change(self, scrollbar):
 		self.correct_coords(self.h_scrollbar.get_value(), self.v_scrollbar.get_value())
@@ -469,7 +507,7 @@ class DrawingImage(Gtk.Box):
 		wanted_x = self.scroll_x + int(delta_x * factor)
 		wanted_y = self.scroll_y + int(delta_y * factor)
 		self.correct_coords(wanted_x, wanted_y)
-		self.window.minimap.update_minimap()
+		self.window.minimap.update_minimap(False)
 
 	def correct_coords(self, wanted_x, wanted_y):
 		available_w = self.get_widget_width()
@@ -507,6 +545,15 @@ class DrawingImage(Gtk.Box):
 		else:
 			self.scroll_x = int(scrollbar.get_value())
 
+	def zoom_to_point(self, delta_x, delta_y, x, y):
+		zoom_delta = (delta_x + delta_y) * -5
+		self.inc_zoom_level(zoom_delta)
+		displayed_w = self.get_widget_width() / self.zoom_level
+		displayed_h = self.get_widget_height() / self.zoom_level
+		fake_delta_x = x - (displayed_w / 2)
+		fake_delta_y = y - (displayed_h / 2)
+		self.add_deltas(fake_delta_x, fake_delta_y, min(1, 1/self.zoom_level))
+
 	def inc_zoom_level(self, delta):
 		self.set_zoom_level((self.zoom_level * 100) + delta)
 
@@ -516,7 +563,9 @@ class DrawingImage(Gtk.Box):
 		self.fake_scrollbar_update()
 		self.update()
 
-	def set_opti_zoom_level(self):
+	def set_opti_zoom_level(self, *args):
+		if not self.zoom_is_auto:
+			return
 		allocated_width = self.get_widget_width()
 		allocated_height = self.get_widget_height()
 		if allocated_width == 1:
