@@ -32,7 +32,6 @@ def main(version):
 ################################################################################
 
 class Application(Gtk.Application):
-	about_dialog = None
 	shortcuts_window = None
 	prefs_window = None
 
@@ -45,7 +44,6 @@ class Application(Gtk.Application):
 		GLib.set_application_name('Drawing') # XXX drawing ?
 		GLib.set_prgname(APP_ID)
 		self.version = version
-		self.git_url = 'https://github.com/maoschanz/drawing'
 		self.has_tools_in_menubar = False
 
 		self.connect('startup', self.on_startup)
@@ -70,8 +68,6 @@ class Application(Gtk.Application):
 		icon_theme.add_resource_path('/com/github/maoschanz/drawing/icons')
 		icon_theme.add_resource_path('/com/github/maoschanz/drawing/tools/icons')
 
-		self.connect('window-removed', self.update_windows_menu_section)
-
 	def on_startup(self, *args):
 		"""Called only once, add app-wide menus and actions, and all accels."""
 		self.build_actions()
@@ -87,7 +83,7 @@ class Application(Gtk.Application):
 		"""Add app-wide actions."""
 		self.add_action_simple('new_window', self.on_new_window, ['<Ctrl>n'])
 		self.add_action_simple('settings', self.on_prefs, None)
-		if self.is_beta():
+		if not self.is_beta():
 			self.add_action_simple('report_bug', self.on_report, None)
 		self.add_action_simple('shortcuts', self.on_shortcuts, \
 		                                         ['<Ctrl>question', '<Ctrl>F1'])
@@ -98,34 +94,9 @@ class Application(Gtk.Application):
 		self.add_action_simple('help_selection', self.on_help_selection, None)
 		self.add_action_simple('about', self.on_about, ['<Shift>F1'])
 		self.add_action_simple('quit', self.on_quit, ['<Ctrl>q'])
-		self.add_action_enum('active-window', 0, self.change_active_win)
 
 	# WINDOWS AND CLI MANAGEMENT ###############################################
 
-	def change_active_win(self, *args):
-		win_id = args[1].get_uint32()
-		for window in self.get_windows():
-			if not isinstance(window, Gtk.ApplicationWindow):
-				continue
-			elif window.get_id() is args[1].get_uint32():
-				window.present()
-		args[0].set_state( GLib.Variant('u', win_id) )
-
-	def update_windows_menu_section(self, *args):
-		# XXX trop appelée à mon goût
-		section = self.get_menubar().get_item_link(6, \
-		          Gio.MENU_LINK_SUBMENU).get_item_link(1, Gio.MENU_LINK_SECTION)
-		section.remove_all()
-		for window in self.get_windows():
-			if not isinstance(window, Gtk.ApplicationWindow):
-				continue
-			elif window.get_title() is None:
-				continue
-			else:
-				detailed_name = 'app.active-window(uint32 ' + \
-				                                      str(window.get_id()) + ')'
-				section.append(window.get_title(), detailed_name)
-
 	def open_window_with_content(self, gfile, get_cb):
 		"""Open a new window with an optional Gio.File as an argument. If get_cb
 		is true, the Gio.File is ignored and the picture is built from the
@@ -137,8 +108,6 @@ class Application(Gtk.Application):
 		# window is presented to the user regarless of loading errors, making
 		# any issue in `init_window_content` very explicit, and more likely to
 		# be reported.
-		action = self.lookup_action('active-window')
-		action.set_state( GLib.Variant('u', win.get_id()) )
 		return win
 
 	def on_activate(self, *args):
@@ -181,13 +150,13 @@ class Application(Gtk.Application):
 
 		elif options.contains('new-window'):
 			self.on_new_window()
-			for path in arguments:
-				f = self.get_valid_file(args[1], path)
+			for fpath in arguments:
+				f = self.get_valid_file(args[1], fpath)
 				if f is not None:
 					self.open_window_with_content(f, False)
 		else: # giving files without '-n' is equivalent to giving files with '-t'
-			for path in arguments:
-				f = self.get_valid_file(args[1], path)
+			for fpath in arguments:
+				f = self.get_valid_file(args[1], fpath)
 				if f is not None:
 					win = self.props.active_window
 					if not win:
@@ -207,7 +176,8 @@ class Application(Gtk.Application):
 	def on_report(self, *args):
 		"""Action callback, opening a new issue on the github repo."""
 		win = self.props.active_window
-		Gtk.show_uri_on_window(win, self.git_url + '/issues/new', Gdk.CURRENT_TIME)
+		url = 'https://github.com/maoschanz/drawing/issues/new'
+		Gtk.show_uri_on_window(win, url, Gdk.CURRENT_TIME)
 
 	def on_shortcuts(self, *args):
 		"""Action callback, showing the "shortcuts" dialog."""
@@ -255,26 +225,18 @@ class Application(Gtk.Application):
 
 	def on_about(self, *args):
 		"""Action callback, showing the "about" dialog."""
-		if self.about_dialog is not None:
-			self.about_dialog.destroy()
-		self.about_dialog = Gtk.AboutDialog(
-			transient_for=self.props.active_window,
-			copyright='© 2019 Romain F. T.',
-			authors=['Romain F. T.'],
+		about_dialog = Gtk.AboutDialog(transient_for=self.props.active_window,
+			copyright='© 2019 Romain F. T.', authors=['Romain F. T.'],
 			# To tranlators: "translate" this by your name, it will be displayed in the "about" dialog
 			translator_credits=_("translator-credits"),
-			artists=['Tobias Bernard'],
+			artists=['Tobias Bernard', 'Romain F. T.'],
 			comments=_("A drawing application for the GNOME desktop."),
 			license_type=Gtk.License.GPL_3_0,
-			logo_icon_name=APP_ID,
-			version=str(self.version),
-			website=self.git_url,
-			website_label=_("Report bugs or ideas"))
-		self.about_dialog.connect('response', self.widget_destroy)
-		self.about_dialog.run()
-
-	def widget_destroy(self, widget, button):
-		widget.destroy()
+			logo_icon_name=APP_ID, version=str(self.version),
+			website='https://maoschanz.github.io/drawing/',
+			website_label=_("Official webpage"))
+		about_dialog.run()
+		about_dialog.destroy()
 
 	def on_quit(self, *args):
 		"""Action callback, quitting the app."""
@@ -305,13 +267,6 @@ class Application(Gtk.Application):
 		action.connect('change-state', callback)
 		self.add_action(action)
 
-	def add_action_enum(self, action_name, default, callback):
-		# XXX attention, celle-ci fonctionne avec un int, pas une string
-		action = Gio.SimpleAction().new_stateful(action_name, \
-		             GLib.VariantType.new('u'), GLib.Variant('u', default))
-		action.connect('change-state', callback)
-		self.add_action(action)
-
 	def get_valid_file(self, app, path):
 		try:
 			f = app.create_file_for_arg(path)
@@ -319,7 +274,7 @@ class Application(Gtk.Application):
 				          Gio.FileQueryInfoFlags.NONE, None).get_content_type():
 				return f
 			else:
-				return None
+				return None # mainly when it's /app/bin/drawing
 		except:
 			err = _("Error opening this file. Did you mean %s ?")
 			command = "\n\tflatpak run --file-forwarding {0} @@ {1} @@\n"
