@@ -23,33 +23,57 @@ from .utilities import utilities_show_overlay_on_context
 class DrawingMinimap(Gtk.Popover):
 	__gtype_name__ = 'DrawingMinimap'
 	# TODO custom "move" cursor
+	# TODO "on_motion" method?
 
 	def __init__(self, window, minimap_btn, **kwargs):
 		super().__init__(**kwargs)
 		self.window = window
-		self.minimap_btn = minimap_btn
 		self.preview_size = self.window._settings.get_int('preview-size')
 		self.mini_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 300, 300)
 		self.mini_surface = cairo.ImageSurface(cairo.Format.ARGB32, 5, 5)
 
-		builder = Gtk.Builder.new_from_resource('/com/github/maoschanz/drawing/ui/minimap.ui')
+		builder = Gtk.Builder.new_from_resource( \
+		                          '/com/github/maoschanz/drawing/ui/minimap.ui')
 		box = builder.get_object('minimap_box')
 
+		self.zoom_scale = builder.get_object('zoom_scale')
+		self.zoom_scale.add_mark(100.0, Gtk.PositionType.TOP, None)
+		self.zoom_scale.connect('value-changed', self.update_zoom_level)
+
 		self.minimap_area = builder.get_object('minimap_area')
 		self.minimap_area.set_size(200, 200)
 		self.minimap_area.add_events(Gdk.EventMask.BUTTON_PRESS_MASK | \
-			Gdk.EventMask.BUTTON_RELEASE_MASK)
+		                                      Gdk.EventMask.BUTTON_RELEASE_MASK)
 		self.minimap_area.connect('draw', self.on_minimap_draw)
 		self.minimap_area.connect('button-press-event', self.on_minimap_press)
 		self.minimap_area.connect('button-release-event', self.on_minimap_release)
 
 		self.add(box)
-		self.set_relative_to(self.minimap_btn)
 		self.connect('closed', self.on_popover_dismissed)
+		self.set_relative_to(minimap_btn)
+
+	def update_zoom_level(self, *args):
+		zoom_value = self.zoom_scale.get_value()
+		self.window.get_active_image().zoom_level = zoom_value/100 # XXX superflu ?
+		self.window.get_active_image().update() # XXX superflu ?
+		self.set_zoom_label(zoom_value)
+
+	def set_zoom_label(self, int_value):
+		# This displays the zoom level, %s will be replaced with a number, while
+		# %% is just the symbol '%'
+		zoom_label = _("%s%%") % str(int(int_value))
+		self.window.options_manager.set_minimap_label(zoom_label)
+		self.update_minimap(False)
+
+	def update_zoom_scale(self, value):
+		self.zoom_scale.set_value(value * 100)
 
 	def on_popover_dismissed(self, *args):
 		"""Callback of the 'closed' signal, updating the state of the action."""
-		self.minimap_btn.set_active(False)
+		try:
+			self.get_relative_to().set_active(False)
+		except:
+			pass
 
 	def on_minimap_draw(self, area, cairo_context):
 		"""Callback of the 'draw' signal, painting the area with the surface."""
@@ -63,42 +87,34 @@ class DrawingMinimap(Gtk.Popover):
 
 	def on_minimap_release(self, area, event):
 		"""Callback of the 'button-release-event' signal."""
-		ratio = self.get_main_pixbuf().get_width()/self.mini_pixbuf.get_width()
-		delta_x = int((event.x - self.old_x) * ratio)
-		delta_y = int((event.y - self.old_y) * ratio)
-		self.window.get_active_image().add_deltas(delta_x, delta_y, 1)
+		image = self.window.get_active_image()
+		size_ratio = image.get_minimap_ratio(self.mini_pixbuf.get_width())
+		delta_x = int((event.x - self.old_x) / size_ratio)
+		delta_y = int((event.y - self.old_y) / size_ratio)
+		image.add_deltas(delta_x, delta_y, 1)
 
-	def get_main_pixbuf(self):
-		return self.window.get_active_image().main_pixbuf
-
-	def update_minimap(self, *args):
+	def update_minimap(self, force_update):
 		"""Update the overlay on the minimap, based on the scroll coordinates."""
-		x = self.window.get_active_image().scroll_x
-		y = self.window.get_active_image().scroll_y
-		w = self.preview_size
-		h = self.preview_size
-		mpb_width = self.get_main_pixbuf().get_width()
-		mpb_height = self.get_main_pixbuf().get_height()
-		if mpb_height > mpb_width:
-			w = self.preview_size * (mpb_width/mpb_height)
-		else:
-			h = self.preview_size * (mpb_height/mpb_width)
-		self.mini_pixbuf = self.get_main_pixbuf().scale_simple(w, h, \
-		                                             GdkPixbuf.InterpType.TILES)
-		self.mini_surface = Gdk.cairo_surface_create_from_pixbuf(self.mini_pixbuf, 0, None)
-		self.minimap_area.set_size_request(self.mini_surface.get_width(), \
-		                                         self.mini_surface.get_height())
+		if not self.get_visible() and not force_update:
+			return
+		image = self.window.get_active_image()
+		self.mini_pixbuf = image.get_mini_pixbuf(self.preview_size)
+		self.mini_surface = Gdk.cairo_surface_create_from_pixbuf( \
+		                                              self.mini_pixbuf, 0, None)
+		pix_width = self.mini_pixbuf.get_width()
+		pix_height = self.mini_pixbuf.get_height()
+		self.minimap_area.set_size_request(pix_width, pix_height)
 
-		visible_width = min(self.window.get_active_image().get_allocated_width(), \
-		                    mpb_width - x)
-		visible_height = min(self.window.get_active_image().get_allocated_height(), \
-		                     mpb_height - y)
-		if self.window.get_active_image().get_allocated_width() < mpb_width \
-		or self.window.get_active_image().get_allocated_height() < mpb_height:
-			mini_x = x * self.mini_pixbuf.get_width()/mpb_width
-			mini_y = y * self.mini_pixbuf.get_height()/mpb_height
-			mini_width = visible_width * self.mini_pixbuf.get_width()/mpb_width
-			mini_height = visible_height * self.mini_pixbuf.get_height()/mpb_height
+		# TODO if possible, updating the overlay should be doable without first
+		# rebuilding the pixbuf and the surface. It's not very useful however
+		# since the guard clause prevent the update in most cases anyway.
+		if image.get_show_overlay():
+			size_ratio = image.get_minimap_ratio(pix_width)
+			mini_x = int(image.scroll_x * size_ratio)
+			mini_y = int(image.scroll_y * size_ratio)
+			visible_width, visible_height = image.get_visible_size()
+			mini_width = int(visible_width * size_ratio)
+			mini_height = int(visible_height * size_ratio)
 			mini_context = cairo.Context(self.mini_surface)
 			mini_context.move_to(mini_x, mini_y)
 			mini_context.line_to(mini_x, mini_height + mini_y)
@@ -110,7 +126,7 @@ class DrawingMinimap(Gtk.Popover):
 		# else:
 		# 	???
 		self.minimap_area.queue_draw()
-		self.update_main_area()
+		image.update()
 
-	def update_main_area(self):
-		self.window.get_active_image().update()
+	############################################################################
+################################################################################
