@@ -16,32 +16,30 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from gi.repository import Gtk, Gio, GLib, Gdk
-from .gi_composites import GtkTemplate
 
 from .utilities import utilities_add_px_to_spinbutton
 
 SETTINGS_SCHEMA = 'com.github.maoschanz.drawing'
 
-@GtkTemplate(ui='/com/github/maoschanz/drawing/ui/preferences.ui')
+@Gtk.Template(resource_path='/com/github/maoschanz/drawing/ui/preferences.ui')
 class DrawingPrefsWindow(Gtk.Window):
 	__gtype_name__ = 'DrawingPrefsWindow'
 
-	content_area = GtkTemplate.Child()
-	stack_switcher = GtkTemplate.Child()
+	content_area = Gtk.Template.Child()
+	stack_switcher = Gtk.Template.Child()
 
-	page_images = GtkTemplate.Child()
-	page_tools = GtkTemplate.Child()
-	page_advanced = GtkTemplate.Child()
+	page_images = Gtk.Template.Child()
+	page_tools = Gtk.Template.Child()
+	page_advanced = Gtk.Template.Child()
 
-	adj_width = GtkTemplate.Child()
-	adj_height = GtkTemplate.Child()
-	adj_preview = GtkTemplate.Child()
+	adj_width = Gtk.Template.Child()
+	adj_height = Gtk.Template.Child()
+	adj_preview = Gtk.Template.Child()
 
 	_settings = Gio.Settings.new('com.github.maoschanz.drawing')
 
 	def __init__(self, is_beta, wants_csd, **kwargs):
 		super().__init__(**kwargs)
-		self.init_template()
 		if wants_csd:
 			header_bar = Gtk.HeaderBar(visible=True, title=_("Preferences"), \
 			                                             show_close_button=True)
@@ -74,22 +72,51 @@ class DrawingPrefsWindow(Gtk.Window):
 		w = self.row_from_widget(_("Default background"), background_color_btn)
 		self.page_images.add(w)
 
-		w = self.row_from_label(_("Opened images"), True)  # TODO move this section to "advanced"
+		w = self.row_from_label(_("Zoom"), True)
 		self.page_images.add(w)
 
-		w = self.row_from_bool(_("Always add transparency"), 'add-alpha')
+		scroll_combobox = Gtk.ComboBoxText()
+		scroll_combobox.append('ctrl', _("Ctrl + Scroll"))
+		scroll_combobox.append('scroll', _("Scroll"))
+		scroll_combobox.set_active_id(self._settings.get_string('zoom-behavior'))
+		scroll_combobox.connect('changed', self.on_combo_changed, 'zoom-behavior')
+		w = self.row_from_widget(_("Action to zoom"), scroll_combobox)
+		self.page_images.add(w)
+
+		w = self.row_from_bool(_("Automatic zoom level"), 'auto-zoom')
 		self.page_images.add(w)
 
 		########################################################################
 		# Build the "tools" page ###############################################
 
 		pass
+		w = self.row_from_label(_("Appearance"), False)
+		self.page_tools.add(w)
+
 		w = self.row_from_bool(_("Show tools names"), 'show-labels')
 		self.page_tools.add(w)
 
 		w = self.row_from_bool(_("Use big icons"), 'big-icons')
 		self.page_tools.add(w)
 
+		w = self.row_from_label(_("Additional tools"), True)
+		self.page_tools.add(w)
+
+		flowbox = Gtk.FlowBox(visible=True, selection_mode=Gtk.SelectionMode.NONE)
+		self.page_tools.add(flowbox)
+
+		w = self.row_for_AoS(_("Free selection"), 'free_select', 'disabled-tools')
+		flowbox.add(w)
+
+		w = self.row_for_AoS(_("Color selection"), 'color_select', 'disabled-tools')
+		flowbox.add(w)
+
+		w = self.row_for_AoS(_("Color Picker"), 'picker', 'disabled-tools')
+		flowbox.add(w)
+
+		w = self.row_for_AoS(_("Paint"), 'paint', 'disabled-tools')
+		flowbox.add(w)
+
 		########################################################################
 		# Build the "advanced" page ############################################
 
