@@ -3,18 +3,18 @@
 from gi.repository import Gtk, Gdk
 import cairo, math
 
-from .abstract_tool import ToolTemplate
+from .abstract_classic_tool import AbstractClassicTool
 
-class ToolExperiment(ToolTemplate):
+class ToolExperiment(AbstractClassicTool):
 	__gtype_name__ = 'ToolExperiment'
 
 	def __init__(self, window, **kwargs):
-		super().__init__('experiment', _("Experiment"), 'applications-science-symbolic', window)
+		super().__init__('experiment', _("Experiment"), 'applications-utilities-symbolic', window)
+		self.use_size = True
+		self.row.get_style_context().add_class('destructive-action')
 		self.past_x = -1.0
 		self.past_y = -1.0
 		self._path = None
-		self.main_color = None
-		self.use_size = True
 
 		self.selected_mode = 'smooth2'
 		self.selected_operator_label = "DIFFERENCE"
@@ -187,18 +187,14 @@ class ToolExperiment(ToolTemplate):
 	def get_edition_status(self):
 		return "You're not supposed to use this tool (development only)."
 
-	def on_press_on_area(self, area, event, surface, tool_width, left_color, right_color, event_x, event_y):
+	def on_press_on_area(self, event, surface, event_x, event_y):
 		self.set_active_operator()
 		self.set_active_mode()
 		self.x_press = event_x
 		self.y_press = event_y
-		self.tool_width = tool_width
-		if event.button == 3:
-			self.main_color = right_color
-		else:
-			self.main_color = left_color
+		self.set_common_values(event)
 
-	def on_motion_on_area(self, area, event, surface, event_x, event_y):
+	def on_motion_on_area(self, event, surface, event_x, event_y):
 		self.restore_pixbuf()
 		cairo_context = cairo.Context(self.get_surface())
 		if self.past_x == -1.0:
@@ -215,7 +211,7 @@ class ToolExperiment(ToolTemplate):
 		operation = self.build_operation()
 		self.do_tool_operation(operation)
 
-	def on_release_on_area(self, area, event, surface, event_x, event_y):
+	def on_release_on_area(self, event, surface, event_x, event_y):
 		self.past_x = -1.0
 		self.past_y = -1.0
 		operation = self.build_operation()
@@ -407,5 +403,6 @@ class ToolExperiment(ToolTemplate):
 		cairo_context.curve_to(nx1, ny1, nx2, ny2, x3, y3)
 		return x1, y1, x2, y2, x3, y3, x4, y4
 
-
+	############################################################################
+################################################################################
 
