@@ -3,10 +3,10 @@
 from gi.repository import Gtk, Gdk
 import cairo
 
-from .abstract_tool import ToolTemplate
+from .abstract_classic_tool import AbstractClassicTool
 from .utilities import utilities_add_arrow_triangle
 
-class ToolLine(ToolTemplate):
+class ToolLine(AbstractClassicTool):
 	__gtype_name__ = 'ToolLine'
 
 	def __init__(self, window, **kwargs):
@@ -18,13 +18,7 @@ class ToolLine(ToolTemplate):
 		self.add_tool_action_boolean('is_arrow', False)
 		self.add_tool_action_boolean('use_gradient', False)
 		self.add_tool_action_enum('cairo_operator', 'over')
-
-		# Default values
-		self.selected_shape_label = _("Round")
-		self.selected_operator = cairo.Operator.OVER
-		self.selected_end_id = cairo.LineCap.ROUND
-		self.use_dashes = False
-		self.use_arrow = False
+		self.set_options_attributes() # Not optimal but more readable
 
 	def set_active_shape(self):
 		if self.get_option_value('line_shape') == 'square':
@@ -34,30 +28,18 @@ class ToolLine(ToolTemplate):
 			self.selected_end_id = cairo.LineCap.ROUND
 			self.selected_shape_label = _("Round")
 
-	def set_active_operator(self):
-		state_as_string = self.get_option_value('cairo_operator')
-		if state_as_string == 'difference':
-			self.selected_operator = cairo.Operator.DIFFERENCE
-			self.selected_operator_label = _("Difference")
-		elif state_as_string == 'source':
-			self.selected_operator = cairo.Operator.SOURCE
-			self.selected_operator_label = _("Source color")
-		elif state_as_string == 'clear':
-			self.selected_operator = cairo.Operator.CLEAR
-			self.selected_operator_label = _("Eraser")
-		else:
-			self.selected_operator = cairo.Operator.OVER
-			self.selected_operator_label = _("Classic")
-
 	def get_options_label(self):
 		return _("Line options")
 
-	def get_edition_status(self): # TODO l'opérateur est important
+	def set_options_attributes(self):
 		self.use_dashes = self.get_option_value('use_dashes')
 		self.use_arrow = self.get_option_value('is_arrow')
 		self.use_gradient = self.get_option_value('use_gradient')
 		self.set_active_shape()
 		self.set_active_operator()
+
+	def get_edition_status(self): # TODO l'opérateur est important
+		self.set_options_attributes()
 		label = self.label + ' (' + self.selected_shape_label + ') '
 		if self.use_arrow and self.use_dashes:
 			label = label + ' - ' + _("Arrow") + ' - ' + _("With dashes")
@@ -71,50 +53,45 @@ class ToolLine(ToolTemplate):
 		self.x_press = 0.0
 		self.y_press = 0.0
 
-	def on_press_on_area(self, area, event, surface, tool_width, left_color, right_color, event_x, event_y):
+	def on_press_on_area(self, event, surface, event_x, event_y):
 		self.x_press = event_x
 		self.y_press = event_y
-		self.tool_width = tool_width
-		if event.button == 1:
-			self.main_color = left_color
-			self.sec_color = right_color
-		if event.button == 3:
-			self.main_color = right_color
-			self.sec_color = left_color
+		self.set_common_values(event)
 
-	def on_motion_on_area(self, area, event, surface, event_x, event_y):
+	def on_motion_on_area(self, event, surface, event_x, event_y):
 		self.restore_pixbuf()
 		cairo_context = cairo.Context(self.get_surface())
 		cairo_context.move_to(self.x_press, self.y_press)
 		cairo_context.line_to(event_x, event_y)
 
 		self._path = cairo_context.copy_path()
-		operation = self.build_operation(event_x, event_y)
+		operation = self.build_operation(event_x, event_y, True)
 		self.do_tool_operation(operation)
 
-	def on_release_on_area(self, area, event, surface, event_x, event_y):
+	def on_release_on_area(self, event, surface, event_x, event_y):
 		self.restore_pixbuf()
 		cairo_context = cairo.Context(self.get_surface())
 		cairo_context.move_to(self.x_press, self.y_press)
 		cairo_context.line_to(event_x, event_y)
 
 		self._path = cairo_context.copy_path()
-		operation = self.build_operation(event_x, event_y)
+		operation = self.build_operation(event_x, event_y, False)
 		self.apply_operation(operation)
 		self.x_press = 0.0
 		self.y_press = 0.0
 
-	def build_operation(self, event_x, event_y):
+	def build_operation(self, event_x, event_y, is_preview):
 		operation = {
 			'tool_id': self.id,
 			'rgba': self.main_color,
-			'rgba2': self.sec_color,
+			'rgba2': self.secondary_color,
 			'operator': self.selected_operator,
 			'line_width': self.tool_width,
 			'line_cap': self.selected_end_id,
 			'use_dashes': self.use_dashes,
 			'use_arrow': self.use_arrow,
 			'use_gradient': self.use_gradient,
+			'is_preview': is_preview,
 			'path': self._path,
 			'x_release': event_x,
 			'y_release': event_y,
@@ -146,7 +123,9 @@ class ToolLine(ToolTemplate):
 		if operation['use_dashes']:
 			cairo_context.set_dash([2*line_width, 2*line_width])
 		cairo_context.append_path(operation['path'])
-		cairo_context.stroke()
+
+		self.stroke_with_operator(operation['operator'], cairo_context, \
+		                                    line_width, operation['is_preview'])
 
 		if operation['use_arrow']:
 			x_press = operation['x_press']
