@@ -18,82 +18,81 @@
 import os
 
 from gi.repository import Gtk, Gdk, Gio, GdkPixbuf, GLib
-from .gi_composites import GtkTemplate
 
 from .tool_arc import ToolArc
-from .tool_circle import ToolCircle
-from .tool_crop import ToolCrop
 from .tool_experiment import ToolExperiment
-from .tool_flip import ToolFlip
-from .tool_freeshape import ToolFreeshape
 from .tool_line import ToolLine
 from .tool_paint import ToolPaint
 from .tool_pencil import ToolPencil
 from .tool_picker import ToolPicker
-from .tool_polygon import ToolPolygon
-from .tool_rectangle import ToolRectangle
-from .tool_rotate import ToolRotate
-from .tool_saturate import ToolSaturate
-from .tool_scale import ToolScale
-from .tool_select import ToolSelect
+from .tool_shape import ToolShape
 from .tool_text import ToolText
 
+from .tool_crop import ToolCrop
+from .tool_matrix import ToolMatrix
+from .tool_filters import ToolFilters
+from .tool_rotate import ToolRotate
+from .tool_scale import ToolScale
+
+from .rect_select import ToolRectSelect
+from .free_select import ToolFreeSelect
+from .color_select import ToolColorSelect
+
 from .image import DrawingImage
 from .properties import DrawingPropertiesDialog
 from .minimap import DrawingMinimap
 from .options_manager import DrawingOptionsManager
-from .color_popover import DrawingColorPopover
 from .message_dialog import DrawingMessageDialog
 from .headerbar import DrawingAdaptativeHeaderBar
 
 from .utilities import utilities_save_pixbuf_at
-from .utilities import utilities_add_px_to_spinbutton
+from .utilities import utilities_add_filechooser_filters
 
 UI_PATH = '/com/github/maoschanz/drawing/ui/'
 
-@GtkTemplate(ui=UI_PATH+'window.ui')
+PLACEHOLDER_UI_STRING = '''<?xml version="1.0"?>
+<interface>
+  <menu id="tool-placeholder">
+    <section>
+      <item>
+        <attribute name="action">none</attribute>
+        <attribute name="label">%s</attribute>
+      </item>
+    </section>
+  </menu>
+</interface>'''
+
+################################################################################
+
+@Gtk.Template(resource_path=UI_PATH+'window.ui')
 class DrawingWindow(Gtk.ApplicationWindow):
 	__gtype_name__ = 'DrawingWindow'
 
 	_settings = Gio.Settings.new('com.github.maoschanz.drawing')
 
 	# Window empty widgets
-	tools_panel = GtkTemplate.Child()
-	toolbar_box = GtkTemplate.Child()
-	info_bar = GtkTemplate.Child()
-	info_label = GtkTemplate.Child()
-	notebook = GtkTemplate.Child()
-	bottom_panel_box = GtkTemplate.Child()
-	tools_scrollable_box = GtkTemplate.Child()
-	tools_nonscrollable_box = GtkTemplate.Child()
-
-	# Default bottom panel
-	bottom_panel = GtkTemplate.Child()
-	color_box = GtkTemplate.Child()
-	color_menu_btn_l = GtkTemplate.Child()
-	color_menu_btn_r = GtkTemplate.Child()
-	l_btn_image = GtkTemplate.Child()
-	r_btn_image = GtkTemplate.Child()
-	thickness_spinbtn = GtkTemplate.Child()
-	options_btn = GtkTemplate.Child()
-	options_label = GtkTemplate.Child()
-	options_long_box = GtkTemplate.Child()
-	options_short_box = GtkTemplate.Child()
-	minimap_btn = GtkTemplate.Child()
-	minimap_icon = GtkTemplate.Child()
-	minimap_label = GtkTemplate.Child()
-	minimap_arrow = GtkTemplate.Child()
+	tools_panel = Gtk.Template.Child()
+	toolbar_box = Gtk.Template.Child()
+	info_bar = Gtk.Template.Child()
+	info_label = Gtk.Template.Child()
+	notebook = Gtk.Template.Child()
+	bottom_panel_box = Gtk.Template.Child()
+	tools_scrollable_box = Gtk.Template.Child()
+	tools_nonscrollable_box = Gtk.Template.Child()
+	fullscreen_btn = Gtk.Template.Child()
+	fullscreen_icon = Gtk.Template.Child()
 
 	def __init__(self, **kwargs):
 		super().__init__(**kwargs)
-		self.init_template()
 		self.app = kwargs['application']
 
 		self.header_bar = None
+		self.fullscreened = False
 
 		if self._settings.get_boolean('maximized'):
 			self.maximize()
 		# self.resize(360, 648) # XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX
+		# self.resize(720, 288) # XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX XXX
 		self.set_ui_bars()
 
 	def init_window_content(self, gfile, get_cb):
@@ -102,12 +101,9 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		parameters, the new image can be imported from the clipboard, loaded
 		from a GioFile, or (else) it can be a blank image."""
 		self.tools = None
-		self.minimap = DrawingMinimap(self, self.minimap_btn)
+		self.minimap = DrawingMinimap(self, None)
 		self.options_manager = DrawingOptionsManager(self)
-		self.thickness_spinbtn.set_value(self._settings.get_int('last-size'))
-		utilities_add_px_to_spinbutton(self.thickness_spinbtn, 3, 'px') # XXX fonctionne mais c'est moche mdr
 
-		self.build_color_buttons()
 		self.add_all_win_actions()
 		if get_cb:
 			self.build_image_from_clipboard()
@@ -116,7 +112,6 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.init_tools()
 		self.connect_signals()
 		self.set_picture_title()
-		self.prompt_message(False, 'window successfully started')
 
 	def init_tools(self):
 		"""Initialize all tools, building the UI for them including the menubar,
@@ -124,39 +119,30 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		disabled_tools_ids = self._settings.get_strv('disabled-tools')
 		dev = self._settings.get_boolean('devel-only')
 		self.tools = {}
+		self.prompt_message(False, 'window has started, now loading tools')
+		# XXX réfléchir à l'ordre ?
 		self.load_tool('pencil', ToolPencil, disabled_tools_ids, dev)
-		self.load_tool('select', ToolSelect, disabled_tools_ids, dev)
+		self.load_tool('rect_select', ToolRectSelect, disabled_tools_ids, dev)
+		self.load_tool('free_select', ToolFreeSelect, disabled_tools_ids, dev)
+		self.load_tool('color_select', ToolColorSelect, disabled_tools_ids, dev)
 		self.load_tool('text', ToolText, disabled_tools_ids, dev)
 		self.load_tool('picker', ToolPicker, disabled_tools_ids, dev)
 		self.load_tool('paint', ToolPaint, disabled_tools_ids, dev)
 		self.load_tool('line', ToolLine, disabled_tools_ids, dev)
 		self.load_tool('arc', ToolArc, disabled_tools_ids, dev)
-		self.load_tool('rectangle', ToolRectangle, disabled_tools_ids, dev)
-		self.load_tool('circle', ToolCircle, disabled_tools_ids, dev)
-		self.load_tool('polygon', ToolPolygon, disabled_tools_ids, dev)
-		self.load_tool('freeshape', ToolFreeshape, disabled_tools_ids, dev)
+		self.load_tool('shape', ToolShape, disabled_tools_ids, dev)
 		if dev:
 			self.load_tool('experiment', ToolExperiment, disabled_tools_ids, dev)
+			self.load_tool('matrix', ToolMatrix, disabled_tools_ids, dev)
 		self.load_tool('crop', ToolCrop, disabled_tools_ids, dev)
 		self.load_tool('scale', ToolScale, disabled_tools_ids, dev)
 		self.load_tool('rotate', ToolRotate, disabled_tools_ids, dev)
-		self.load_tool('flip', ToolFlip, disabled_tools_ids, dev)
-		self.load_tool('saturate', ToolSaturate, disabled_tools_ids, dev)
+		self.load_tool('filters', ToolFilters, disabled_tools_ids, dev)
 
 		# Side panel buttons for tools, and their menu-items if they don't exist
 		self.build_tool_rows()
 		if not self.app.has_tools_in_menubar:
-			drawing_tools_section = self.app.get_menubar().get_item_link(4, \
-			      Gio.MENU_LINK_SUBMENU).get_item_link(0, Gio.MENU_LINK_SECTION)
-			canvas_tools_section = self.app.get_menubar().get_item_link(4, \
-			    Gio.MENU_LINK_SUBMENU).get_item_link(1, Gio.MENU_LINK_SECTION).get_item_link(0, \
-			      Gio.MENU_LINK_SUBMENU).get_item_link(0, Gio.MENU_LINK_SECTION)
-			for tool_id in self.tools:
-				if self.tools[tool_id].menu_id > 0:
-					self.tools[tool_id].add_item_to_menu(canvas_tools_section)
-				else:
-					self.tools[tool_id].add_item_to_menu(drawing_tools_section)
-			self.app.has_tools_in_menubar = True
+			self.build_menubar_tools_menu()
 
 		# Initialisation of options and menus
 		tool_id = self._settings.get_string('last-active-tool')
@@ -180,6 +166,30 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			except:
 				self.prompt_message(True, _("Failed to load tool: %s") % tool_id)
 
+	def build_tool_rows(self):
+		"""Adds each tool's button to the side panel."""
+		group = None
+		for tool_id in self.tools:
+			if group is None:
+				group = self.tools[tool_id].row
+			else:
+				self.tools[tool_id].row.join_group(group)
+			self.tools_panel.add(self.tools[tool_id].row)
+		self.on_show_labels_setting_changed()
+
+	def build_menubar_tools_menu(self):
+		selection_tools_section = self.get_menubar_item([[True, 4], [False, 0]])
+		drawing_tools_section = self.get_menubar_item([[True, 4], [False, 1]])
+		canvas_tools_section = self.get_menubar_item([[True, 4], [False, 2]])
+		for tool_id in self.tools:
+			if self.tools[tool_id].menu_id == 0:
+				self.tools[tool_id].add_item_to_menu(drawing_tools_section)
+			elif self.tools[tool_id].menu_id == 1:
+				self.tools[tool_id].add_item_to_menu(canvas_tools_section)
+			elif self.tools[tool_id].menu_id == 2:
+				self.tools[tool_id].add_item_to_menu(selection_tools_section)
+		self.app.has_tools_in_menubar = True
+
 	############################################################################
 	# TABS AND WINDOWS MANAGEMENT ##############################################
 
@@ -217,11 +227,11 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.change_active_tool_for(self.active_tool_id)
 		# On devrait être moins bourrin et conserver la sélection #FIXME
 		self.set_picture_title(args[1].update_title())
+		self.minimap.set_zoom_label(args[1].zoom_level * 100)
 
 	def update_tabs_menu_section(self, *args):
 		action = self.lookup_action('active_tab')
-		section = self.app.get_menubar().get_item_link(2, \
-		          Gio.MENU_LINK_SUBMENU).get_item_link(6, Gio.MENU_LINK_SECTION)
+		section = self.get_menubar_item([[True, 2], [False, 6]])
 		section.remove_all()
 		for page in self.notebook.get_children():
 			tab_title = page.update_title()
@@ -256,12 +266,12 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			if not self.get_active_image().try_close_tab():
 				return True
 
-		self._settings.set_int('last-size', int(self.thickness_spinbtn.get_value()))
+		self._settings.set_int('last-size', self.options_manager.get_tool_width())
 		self._settings.set_string('last-active-tool', self.active_tool_id)
-		rgba = self.color_popover_l.color_widget.get_rgba()
+		rgba = self.options_manager.get_left_color()
 		rgba = [str(rgba.red), str(rgba.green), str(rgba.blue), str(rgba.alpha)]
 		self._settings.set_strv('last-left-rgba', rgba)
-		rgba = self.color_popover_r.color_widget.get_rgba()
+		rgba = self.options_manager.get_right_color()
 		rgba = [str(rgba.red), str(rgba.green), str(rgba.blue), str(rgba.alpha)]
 		self._settings.set_strv('last-right-rgba', rgba)
 
@@ -276,12 +286,29 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.info_bar.connect('response', self.hide_message)
 		self.connect('delete-event', self.on_close)
 		self.connect('configure-event', self.adapt_to_window_size)
-		self.options_btn.connect('toggled', self.update_option_label)
 		self._settings.connect('changed::show-labels', self.on_show_labels_setting_changed)
 		self._settings.connect('changed::decorations', self.on_layout_changed)
 		self._settings.connect('changed::big-icons', self.on_icon_size_changed)
 		self.notebook.connect('switch-page', self.on_active_tab_changed)
 
+		self.notebook.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.MOVE)
+		self.notebook.connect('drag-data-received', self.on_data_dropped)
+		self.notebook.drag_dest_add_uri_targets()
+		# because drag_dest_add_image_targets doesn't work for files
+
+	def get_menubar_item(self, path_description):
+		"""Get an item of the app-wide menubar. The `path_description` object
+		is an array of [boolean, int] couples. The boolean means if we're
+		looking for a submenu, the int is an index."""
+		current_item = self.app.get_menubar()
+		for item in path_description:
+			if item[0]:
+				link_type = Gio.MENU_LINK_SUBMENU
+			else:
+				link_type = Gio.MENU_LINK_SECTION
+			current_item = current_item.get_item_link(item[1], link_type)
+		return current_item
+
 	def add_action_simple(self, action_name, callback, shortcuts):
 		"""Convenient wrapper method adding a stateless action to the window. It
 		will be named 'action_name' (string) and activating the action will
@@ -318,6 +345,8 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('main_menu', self.action_main_menu, ['F10'])
 		self.add_action_simple('options_menu', self.action_options_menu, ['<Shift>F10'])
 		self.add_action_simple('properties', self.action_properties, None)
+		self.add_action_simple('fullscreen', self.action_fullscreen, ['F11'])
+		self.add_action_simple('unfullscreen', self.action_unfullscreen, ['Escape'])
 
 		self.add_action_boolean('toggle_preview', False, self.action_toggle_preview)
 		self.app.set_accels_for_action('win.toggle_preview', ['<Ctrl>m'])
@@ -330,6 +359,11 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.add_action_simple('go_left', self.action_go_left, ['<Ctrl>Left'])
 		self.add_action_simple('go_right', self.action_go_right, ['<Ctrl>Right'])
 
+		self.add_action_simple('zoom_in', self.action_zoom_in, ['<Ctrl>plus', '<Ctrl>KP_Add'])
+		self.add_action_simple('zoom_out', self.action_zoom_out, ['<Ctrl>minus', '<Ctrl>KP_Subtract'])
+		self.add_action_simple('zoom_100', self.action_zoom_100, ['<Ctrl>1', '<Ctrl>KP_1'])
+		self.add_action_simple('zoom_opti', self.action_zoom_opti, ['<Ctrl>0', '<Ctrl>KP_0'])
+
 		self.add_action_simple('new_tab', self.build_new_image, ['<Ctrl>t'])
 		self.add_action_simple('new_tab_selection', \
 		                    self.build_image_from_selection, ['<Ctrl><Shift>t'])
@@ -416,8 +450,6 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		subtitle = self.active_tool().get_edition_status()
 
 		self.update_tabs_menu_section()
-		# Uselessly idiot i guess
-		self.app.update_windows_menu_section()
 
 		self.set_title(_("Drawing") + ' - ' + main_title + ' - ' + subtitle)
 		if self.header_bar is not None:
@@ -449,23 +481,12 @@ class DrawingWindow(Gtk.ApplicationWindow):
 	def set_ui_bars(self):
 		"""Set the UI "bars" (headerbar, menubar, title, toolbar, whatever)
 		according to the user's preference, which by default is 'auto'."""
-		self.has_good_limits = False
-		self.limit_size_bottom = 600
+		self.has_good_limits = False # used by the bottom panels
 
+		builder = Gtk.Builder.new_from_string(PLACEHOLDER_UI_STRING \
+		                                                  % _("No options"), -1)
 		# Loading a whole file in a GtkBuilder just for this looked ridiculous,
 		# so it's built from a string.
-		builder = Gtk.Builder.new_from_string('''
-<?xml version="1.0"?>
-<interface>
-  <menu id="tool-placeholder">
-    <section>
-      <item>
-        <attribute name="action">none</attribute>
-        <attribute name="label">''' + _("No options") + '''</attribute>
-      </item>
-    </section>
-  </menu>
-</interface>''', -1)
 		self.placeholder_model = builder.get_object('tool-placeholder')
 
 		# Remember the setting, so no need to restart this at each dialog.
@@ -503,6 +524,17 @@ class DrawingWindow(Gtk.ApplicationWindow):
 
 		if self.app.is_beta():
 			self.get_style_context().add_class('devel')
+		self.set_fullscreen_menu()
+
+	def set_fullscreen_menu(self):
+		builder = Gtk.Builder.new_from_resource(UI_PATH + 'win-menus.ui')
+		fullscreen_menu = builder.get_object('fullscreen-menu')
+		tabs_list = self.get_menubar_item([[True, 2], [False, 6]])
+		fullscreen_menu.append_section(_("Opened images"), tabs_list)
+		tools_menu = self.get_menubar_item([[True, 4]])
+		section = fullscreen_menu.get_item_link(3, Gio.MENU_LINK_SECTION)
+		section.prepend_submenu(_("_Tools"), tools_menu)
+		self.fullscreen_btn.set_menu_model(fullscreen_menu)
 
 	def build_toolbar(self, symbolic):
 		if symbolic:
@@ -535,7 +567,9 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.header_bar = DrawingAdaptativeHeaderBar(is_eos)
 
 	def action_main_menu(self, *args):
-		if self.header_bar is not None:
+		if self.fullscreened:
+			self.fullscreen_btn.set_active(not self.fullscreen_btn.get_active())
+		elif self.header_bar is not None:
 			self.header_bar.toggle_menu()
 
 	def action_options_menu(self, *args):
@@ -543,52 +577,26 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		action to ease the accelerator (shift+f10). This action could be
 		disable when the current panel doesn't contain the corresponding button,
 		but will not be."""
-		if not self.active_tool().implements_panel:
-			self.options_btn.set_active(not self.options_btn.get_active())
-
-	def set_bottom_width_limit(self): # XXX devrait se transmettre aux panneaux custom
-		if self.has_good_limits: # heureusement ils marchent assez bien tous seuls
-			return
-		self.bottom_panel.show_all() # XXX vraiment ?
-		self.limit_size_bottom = self.color_box.get_preferred_width()[0] + \
-		                 self.thickness_spinbtn.get_preferred_width()[0] + \
-		                  self.options_long_box.get_preferred_width()[0] + \
-		                     self.minimap_label.get_preferred_width()[0]
-		self.bottom_panel.set_visible(not self.active_tool().implements_panel)
-
-	def init_adaptability(self):
-		"""Initialize limit_size_header and limit_size_bottom, which are 700 by
-		default, but are likely to actually be less wide."""
-		self.set_bottom_width_limit()
-		if self.header_bar is not None:
-			self.header_bar.init_adaptability()
-		self.has_good_limits = True
+		self.options_manager.toggle_menu()
 
 	def adapt_to_window_size(self, *args):
 		"""Adapts the headerbar (if any) and the default bottom panel to the new
 		window size. If the current bottom panel isn't the default one, this
 		will call the tool method applying the new size to the tool panel."""
 		if not self.has_good_limits and self.get_allocated_width() > 700:
-			self.init_adaptability()
+			self.options_manager.init_adaptability()
+			if self.header_bar is not None:
+				self.header_bar.init_adaptability()
+			self.has_good_limits = True
 
 		if self.header_bar is not None:
 			self.header_bar.adapt_to_window_size()
 
 		available_width = self.bottom_panel_box.get_allocated_width()
-		for tool_id in self.tools:
-			self.tools[tool_id].adapt_to_window_size(available_width)
-		should_shrink = self.limit_size_bottom > 0.7 * available_width
-		self.compact_bottombar(should_shrink)
+		self.options_manager.adapt_to_window_size(available_width)
 
 		self.get_active_image().fake_scrollbar_update()
 
-	def compact_bottombar(self, state):
-		self.options_long_box.set_visible(not state)
-		self.minimap_label.set_visible(not state)
-		self.minimap_arrow.set_visible(not state)
-		self.options_short_box.set_visible(state)
-		self.minimap_icon.set_visible(state)
-
 	def hide_message(self, *args):
 		self.prompt_message(False, '')
 
@@ -601,7 +609,28 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			print('Drawing: ' + label)
 
 	def update_tabs_visibility(self):
-		self.notebook.set_show_tabs(self.notebook.get_n_pages() > 1)
+		should_show = (self.notebook.get_n_pages() > 1) and not self.fullscreened
+		self.notebook.set_show_tabs(should_show)
+
+	############################################################################
+	# FULLSCREEN ###############################################################
+
+	def action_unfullscreen(self, *args):
+		# TODO connect to signals instead
+		self.unfullscreen()
+		self.set_fullscreen_state(False)
+
+	def action_fullscreen(self, *args):
+		# TODO connect to signals instead?
+		self.fullscreen()
+		self.set_fullscreen_state(True)
+
+	def set_fullscreen_state(self, state):
+		self.fullscreened = state
+		self.tools_panel.set_visible(not state)
+		self.toolbar_box.set_visible(not state) # XXX not if not empty
+		self.fullscreen_btn.set_visible(state)
+		self.update_tabs_visibility()
 
 	############################################################################
 	# TOOLS PANEL ##############################################################
@@ -610,18 +639,6 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		for tool_id in self.tools:
 			self.tools[tool_id].update_icon_size()
 
-	def build_tool_rows(self):
-		"""Adds each tool's button to the side panel, or to the selection's
-		bottom panel."""
-		group = None
-		for tool_id in self.tools:
-			if group is None:
-				group = self.tools[tool_id].row
-			else:
-				self.tools[tool_id].row.join_group(group)
-			self.tools_panel.add(self.tools[tool_id].row)
-		self.on_show_labels_setting_changed()
-
 	def set_tools_labels_visibility(self, visible):
 		"""Change the way tools are displayed in the side panel. Visible labels
 		mean the tools will be arranged in a scrollable list of buttons, else
@@ -672,30 +689,38 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			self.tools[tool_id].row.set_active(True)
 
 	def enable_tool(self, new_tool_id):
-		"""Deactivate the formerly active tool, and activate `new_tool_id`."""
-		self.former_tool_id = self.active_tool_id
-		should_preserve_selection = self.tools[new_tool_id].accept_selection
-		self.former_tool().give_back_control(should_preserve_selection)
-		self.former_tool().on_tool_unselected()
-		self.get_active_image().selection.show_popover(False)
+		"""Activate the tool whose id is `new_tool_id`."""
+		self.disable_former_tool(new_tool_id)
 		self.get_active_image().update()
 		self.active_tool_id = new_tool_id
-		self.update_bottom_panel()
-		self.color_box.set_sensitive(self.active_tool().use_color)
 		self.active_tool().on_tool_selected()
+		self.update_fullscreen_icon()
+		self.update_bottom_panel()
 		self.get_active_image().update_actions_state()
 		self.set_picture_title()
 
+	def disable_former_tool(self, future_tool_id):
+		"""Unactivate the formerly active tool."""
+		self.former_tool_id = self.active_tool_id
+		should_preserve_selection = self.tools[future_tool_id].accept_selection
+		self.former_tool().give_back_control(should_preserve_selection)
+		self.former_tool().on_tool_unselected()
+		self.get_active_image().selection.show_popover(False)
+
 	def update_bottom_panel(self):
 		"""Show the correct bottom panel, with the correct tool options menu."""
+		self.options_manager.try_enable_panel(self.active_tool().panel_id)
+		self.options_manager.update_panel(self.active_tool())
 		self.build_options_menu()
-		if self.former_tool_id is not self.active_tool_id:
-			self.former_tool().show_panel(False)
-		self.active_tool().show_panel(True)
-		self.set_bottom_width_limit()
-		self.update_thickness_spinbtn_state()
 		self.adapt_to_window_size()
 
+	def update_fullscreen_icon(self):
+		"""Show the icon of the currently active tool on the button managing
+		fullscreen's main menu."""
+		name = self.active_tool().icon_name
+		img = Gtk.Image.new_from_icon_name(name, Gtk.IconSize.BUTTON)
+		self.fullscreen_btn.set_image(img)
+
 	def active_tool(self):
 		return self.tools[self.active_tool_id]
 
@@ -705,6 +730,48 @@ class DrawingWindow(Gtk.ApplicationWindow):
 	def back_to_previous(self, *args):
 		self.tools[self.former_tool_id].row.set_active(True)
 
+	def active_panel(self):
+		return self.options_manager.get_active_panel()
+
+	def build_options_menu(self):
+		"""Build the active tool's option menus.
+		The first menu is the popover from the bottom bar. It can contain any
+		widget, or it can be build from a Gio.MenuModel
+		The second menu is build from a Gio.MenuModel and is in the menubar (not
+		available with all layouts)."""
+		widget = self.active_tool().get_options_widget()
+		model = self.active_tool().get_options_model()
+		label = self.active_tool().get_options_label()
+		if model is None:
+			self.app.get_menubar().remove(5)
+			self.app.get_menubar().insert_submenu(5, _("_Options"), self.placeholder_model)
+		else:
+			self.app.get_menubar().remove(5)
+			self.app.get_menubar().insert_submenu(5, _("_Options"), model)
+		panel = self.active_panel()
+		if panel is not None:
+			panel.build_options_menu(widget, model, label)
+		else:
+			self.prompt_message(True, 'panel is none for label:' + label)
+
+	def action_use_editor(self, *args):
+		use_editor = not args[0].get_state()
+		self._settings.set_boolean('direct-color-edit', use_editor)
+		args[0].set_state(GLib.Variant.new_boolean(use_editor))
+		self.options_manager.set_palette_setting(use_editor)
+
+	def on_middle_click(self, event):
+		self.exchange_colors()
+
+	def exchange_colors(self, *args):
+		self.options_manager.exchange_colors()
+
+	def action_color1(self, *args):
+		self.options_manager.left_color_btn().activate()
+
+	def action_color2(self, *args):
+		self.options_manager.right_color_btn().activate()
+
 	############################################################################
 	# IMAGE FILES MANAGEMENT ###################################################
 
@@ -753,13 +820,47 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		gfile = None
 		file_chooser = Gtk.FileChooserNative.new(_("Open a picture"), self,
 		                     Gtk.FileChooserAction.OPEN, _("Open"), _("Cancel"))
-		self.add_filechooser_filters(file_chooser)
+		utilities_add_filechooser_filters(file_chooser)
 		response = file_chooser.run()
 		if response == Gtk.ResponseType.ACCEPT:
 			gfile = file_chooser.get_file()
 		file_chooser.destroy()
 		return gfile
 
+	def on_data_dropped(self, widget, drag_context, x, y, data, info, time):
+		"""Signal callback: when files are dropped on `self.notebook`, a message
+		dialog is shown, asking if the user prefers to open them (one new tab
+		per image), or to import them (it will only import the first), or to
+		cancel (if the user dropped mistakenly)."""
+		dialog = DrawingMessageDialog(self)
+		cancel_id = dialog.set_action(_("Cancel"), None, False)
+		open_id = dialog.set_action(_("Open"), None, False)
+		import_id = dialog.set_action(_("Import"), None, True)
+		uris = data.get_uris()
+		if len(uris) == 1:
+			label = uris[0].split('/')[-1]
+		else:
+			label = _("these files")
+		dialog.add_string(_("What do you want to do with %s?") % label)
+		result = dialog.run()
+		dialog.destroy()
+		for uri in uris:
+			# print(uri)
+			# valider l'URI TODO
+			if result == import_id:
+				f = Gio.File.new_for_uri(uri)
+				self.import_from_path(f.get_path())
+				return
+			elif result == open_id:
+				f = Gio.File.new_for_uri(uri)
+				self.build_new_tab(f, None)
+
+	def try_load_file(self, gfile):
+		if gfile is not None:
+			self.get_active_image().try_load_file(gfile)
+		self.set_picture_title() # often redundant but not useless
+		self.prompt_message(False, 'file successfully loaded')
+
 	def action_save(self, *args):
 		"""Try to save the active image, and return True if the image has been
 		successfully saved."""
@@ -783,12 +884,6 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			self.get_active_image().gfile = gfile
 			self.action_save()
 
-	def try_load_file(self, gfile):
-		if gfile is not None:
-			self.get_active_image().try_load_file(gfile)
-		self.set_picture_title() # often redundant but not useless
-		self.prompt_message(False, 'file successfully loaded')
-
 	def confirm_save_modifs(self):
 		"""Return True if the image can be closed/overwritten (whether it's saved
 		or not), or False if the image can't be closed/overwritten."""
@@ -807,7 +902,7 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		cancel_id = dialog.set_action(_("Cancel"), None, False)
 		save_id = dialog.set_action(_("Save"), None, True)
 		dialog.add_string( _("There are unsaved modifications to %s.") % display_name)
-		self.minimap.update_minimap()
+		self.minimap.update_minimap(True)
 		image = Gtk.Image().new_from_pixbuf(self.minimap.mini_pixbuf)
 		frame = Gtk.Frame(valign=Gtk.Align.CENTER, halign=Gtk.Align.CENTER)
 		frame.add(image)
@@ -821,37 +916,12 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		else: # cancel_id
 			return False
 
-	def add_filechooser_filters(self, dialog):
-		"""Add file filters for images to file chooser dialogs."""
-		allPictures = Gtk.FileFilter()
-		allPictures.set_name(_("All pictures"))
-		allPictures.add_mime_type('image/png')
-		allPictures.add_mime_type('image/jpeg')
-		allPictures.add_mime_type('image/bmp')
-
-		pngPictures = Gtk.FileFilter()
-		pngPictures.set_name(_("PNG images"))
-		pngPictures.add_mime_type('image/png')
-
-		jpegPictures = Gtk.FileFilter()
-		jpegPictures.set_name(_("JPEG images"))
-		jpegPictures.add_mime_type('image/jpeg')
-
-		bmpPictures = Gtk.FileFilter()
-		bmpPictures.set_name(_("BMP images"))
-		bmpPictures.add_mime_type('image/bmp')
-
-		dialog.add_filter(allPictures)
-		dialog.add_filter(pngPictures)
-		dialog.add_filter(jpegPictures)
-		dialog.add_filter(bmpPictures)
-
 	def file_chooser_save(self):
 		"""Opens an "save" file chooser dialog, and return a GioFile or None."""
 		gfile = None
 		file_chooser = Gtk.FileChooserNative.new(_("Save picture as…"), self,
 		                     Gtk.FileChooserAction.SAVE, _("Save"), _("Cancel"))
-		self.add_filechooser_filters(file_chooser)
+		utilities_add_filechooser_filters(file_chooser)
 		default_file_name = str(_("Untitled") + '.png')
 		file_chooser.set_current_name(default_file_name)
 		response = file_chooser.run()
@@ -888,7 +958,6 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		self.action_delete()
 
 	def action_copy(self, *args):
-		# self.get_active_image().selection.selection_has_been_used = True # XXX bizarre
 		self.copy_operation()
 
 	def copy_operation(self):
@@ -911,27 +980,36 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			self.active_tool().on_release_on_area(None, None, None, 100, 100)
 
 	def action_import(self, *args):
-		"""Handle the result of an "open" file chooser dialog, and import it
-		as the selection."""
+		"""Handle the result of an "open" file chooser dialog, and it will try
+		to import it as the selection."""
 		file_chooser = Gtk.FileChooserNative.new(_("Import a picture"), self,
 		                   Gtk.FileChooserAction.OPEN, _("Import"), _("Cancel"))
-		self.add_filechooser_filters(file_chooser)
+		utilities_add_filechooser_filters(file_chooser)
 		response = file_chooser.run()
 		if response == Gtk.ResponseType.ACCEPT:
-			self.force_selection()
-			fn = file_chooser.get_filename()
-			self.get_selection_tool().import_selection(GdkPixbuf.Pixbuf.new_from_file(fn))
+			self.import_from_path(file_chooser.get_filename())
 		file_chooser.destroy()
 
+	def import_from_path(self, file_path):
+		"""Import a file as the selection pixbuf. Called by the 'win.import'
+		action and when an image is imported by drag-and-drop."""
+		self.force_selection()
+		pixbuf = GdkPixbuf.Pixbuf.new_from_file(file_path)
+		self.get_selection_tool().import_selection(pixbuf)
+
 	def action_selection_export(self, *args):
 		gfile = self.file_chooser_save()
 		if gfile is not None:
 			pixbuf = self.get_active_image().selection.get_pixbuf()
 			utilities_save_pixbuf_at(pixbuf, gfile.get_path())
 
-	def get_selection_tool(self):  # XXX réellement utile ?
-		if 'select' in self.tools:
-			return self.tools['select']
+	def get_selection_tool(self):
+		if 'rect_select' in self.tools:
+			return self.tools['rect_select']
+		elif 'free_select' in self.tools:
+			return self.tools['free_select']
+		elif 'color_select' in self.tools:
+			return self.tools['color_select']
 		else:
 			self.prompt_message(True, 'Required tool is not available')
 			return self.active_tool()
@@ -947,12 +1025,10 @@ class DrawingWindow(Gtk.ApplicationWindow):
 
 	def action_undo(self, *args):
 		self.get_active_image().try_undo()
-		self.action_rebuild()
 		# self.get_active_image().reset_temp() # XXX ne devrait pas exister
 
 	def action_redo(self, *args):
 		self.get_active_image().try_redo()
-		self.action_rebuild()
 		# self.get_active_image().reset_temp() # XXX ne devrait pas exister
 
 	def operation_is_ongoing(self): # TODO
@@ -969,13 +1045,7 @@ class DrawingWindow(Gtk.ApplicationWindow):
 
 	def action_rebuild(self, *args): # XXX image method ?
 		"""Rebuild the image according to the history content."""
-		self.get_active_image().restore_first_pixbuf()
-		h = self.get_active_image().undo_history.copy()
-		self.get_active_image().undo_history = []
-		for op in h:
-			self.tools[op['tool_id']].apply_operation(op)
-		self.get_active_image().update()
-		self.get_active_image().update_history_sensitivity()
+		self.get_active_image().rebuild_from_history()
 
 	def update_history_actions_labels(self, undo_label, redo_label):
 		# TODO menubar
@@ -983,72 +1053,6 @@ class DrawingWindow(Gtk.ApplicationWindow):
 			self.header_bar.set_undo_label(undo_label)
 			self.header_bar.set_redo_label(redo_label)
 
-	############################################################################
-	# COLORS ###################################################################
-
-	def build_color_buttons(self):
-		"""Initialize the 2 color buttons and popovers with the 2 previously
-		memorized RGBA values."""
-		right_rgba = self._settings.get_strv('last-right-rgba')
-		left_rgba = self._settings.get_strv('last-left-rgba')
-		self.color_popover_r = DrawingColorPopover(self.color_menu_btn_r, \
-		                                           self.r_btn_image, right_rgba)
-		self.color_popover_l = DrawingColorPopover(self.color_menu_btn_l, \
-		                                           self.l_btn_image, left_rgba)
-
-	def action_use_editor(self, *args):
-		use_editor = not args[0].get_state()
-		self._settings.set_boolean('direct-color-edit', use_editor)
-		args[0].set_state(GLib.Variant.new_boolean(use_editor))
-		self.set_palette_setting()
-
-	def set_palette_setting(self, *args):
-		show_editor = self._settings.get_boolean('direct-color-edit')
-		self.color_popover_r.setting_changed(show_editor)
-		self.color_popover_l.setting_changed(show_editor)
-
-	def action_color1(self, *args):
-		self.color_menu_btn_l.activate()
-
-	def action_color2(self, *args):
-		self.color_menu_btn_r.activate()
-
-	def exchange_colors(self, *args):
-		left_c = self.color_popover_l.color_widget.get_rgba()
-		self.color_popover_l.color_widget.set_rgba(self.color_popover_r.color_widget.get_rgba())
-		self.color_popover_r.color_widget.set_rgba(left_c)
-
-	############################################################################
-	# TOOLS OPTIONS ############################################################
-
-	def update_thickness_spinbtn_state(self):
-		self.thickness_spinbtn.set_sensitive(self.active_tool().use_size)
-
-	def build_options_menu(self):
-		"""Build the active tool's option menus.
-		The first menu is the popover from the bottom bar. It can contain any
-		widget, or it can be build from a Gio.MenuModel
-		The second menu is build from a Gio.MenuModel and is in the menubar (not
-		available with all layouts)."""
-		widget = self.active_tool().get_options_widget()
-		model = self.active_tool().get_options_model()
-		if model is None:
-			self.app.get_menubar().remove(5)
-			self.app.get_menubar().insert_submenu(5, _("_Options"), self.placeholder_model)
-		else:
-			self.app.get_menubar().remove(5)
-			self.app.get_menubar().insert_submenu(5, _("_Options"), model)
-		if widget is not None:
-			self.options_btn.set_popover(widget)
-		elif model is not None:
-			self.options_btn.set_menu_model(model)
-		else:
-			self.options_btn.set_popover(None)
-		self.update_option_label()
-
-	def update_option_label(self, *args):
-		self.options_label.set_label(self.active_tool().get_options_label())
-
 	############################################################################
 	# PREVIEW, NAVIGATION AND ZOOM ACTIONS #####################################
 
@@ -1057,7 +1061,7 @@ class DrawingWindow(Gtk.ApplicationWindow):
 		preview_visible = not args[0].get_state()
 		if preview_visible:
 			self.minimap.popup()
-			self.minimap.update_minimap()
+			self.minimap.update_minimap(True)
 		else:
 			self.minimap.popdown()
 		args[0].set_state(GLib.Variant.new_boolean(preview_visible))
@@ -1074,6 +1078,18 @@ class DrawingWindow(Gtk.ApplicationWindow):
 	def action_go_right(self, *args):
 		self.get_active_image().add_deltas(1, 0, 100)
 
+	def action_zoom_in(self, *args):
+		self.get_active_image().inc_zoom_level(50)
+
+	def action_zoom_out(self, *args):
+		self.get_active_image().inc_zoom_level(-50)
+
+	def action_zoom_100(self, *args):
+		self.get_active_image().set_zoom_level(100)
+
+	def action_zoom_opti(self, *args):
+		self.get_active_image().set_opti_zoom_level()
+
 	############################################################################
 ################################################################################
 
